class ErrorMessages:
    # security
    NO_PASSWORD = 'Please enter a password.'
    INVALID_EMAIL = 'Email address is invalid.'
    NO_USERNAME = 'Please enter an email address.'
    NOT_UNIQUE_EMAIL = 'This email is already registered. Sign in.'
    NO_FB_AUTH_TOKEN = 'Facebook token missing.'
    INVALID_FB_AUTH_TOKEN = 'Invalid or expired Facebook token.'
    MISSING_USERNAME = 'Please enter a valid e-mail address.'
    INVALID_USERNAME = 'E-mail address does not exist in the system.'
    INVALID_RESET_PASSWORD_TOKEN = 'Reset password link is expired or invalid.'
    ACCOUNT_DISABLED = 'User account is disabled.'
    INVALID_CREDENTIALS = 'Unable to log in with provided credentials.'
    MISSING_USERNAME_OR_PASSWORD = 'Must include username and password.'
    TOO_MANY_INCORRECT_LOGIN_ATTEMPTS = 'Too many login attempts. Please try again in 20 minutes.'
    PERMISSION_DENIED = 'You do not have the permission to perform this action.'
    INVALID_API_KEY = 'Error 1001 API key not found'
    MISSING_API_KEY = 'Error 1002 No API key provided'
    REPLAY_ERROR = 'Error 1003'
    INVALID_NONCE = 'Error 1004 Provided nonce does not match computed nonce'
    MISSING_NONCE = 'Error 1005 No Nonce provided'
    NO_SSO_TOKEN_CLASSES_DEFINED = 'Misconfigured SSO Token Classes'
    NO_SSO_TOKEN_FOUND = 'No SSO token found - please regenerate'
    API_KEY_NOT_AUTHORIZED_FOR_ENDPOINT = 'ApiKey is not authorized for endpoint {0}'

